@@ -6,6 +6,11 @@ namespace Imendio.Blam
 {
     public partial class AddChannelDialog : Gtk.Dialog
     {
+		bool OkSensitive()
+		{
+			return !string.IsNullOrEmpty(Url);
+		}
+
         public AddChannelDialog(Gtk.Window window)
         {
             this.Build();
@@ -20,11 +25,12 @@ namespace Imendio.Blam
                 }
             });
 
+            buttonOk.Sensitive = OkSensitive();
             UrlEntry.Activated += OnEntryActivated;
             UsernameEntry.Activated += OnEntryActivated;
             PasswordEntry.Activated += OnEntryActivated;
 
-            UrlEntry.Changed += (sender, e) => buttonOk.Sensitive = !String.IsNullOrEmpty(Url);
+            UrlEntry.Changed += (sender, e) => buttonOk.Sensitive = OkSensitive();
         }
 
         public string Username {
