@@ -17,6 +17,8 @@ using System.Collections;
 using System.IO;
 using System.Text;
 using System.Runtime.InteropServices;
+using System.Threading;
+using System.Threading.Tasks;
 using WebKit;
 #if ENABLE_DBUS
 using DBus;
@@ -91,6 +93,7 @@ namespace Imendio.Blam {
         private TrayIcon    trayIcon;
 
         public static string BaseDir;
+        public static SynchronizationContext Context { get; private set; }
 
         private ChannelDialog     channelDialog;
         private AddGroupDialog    addGroupDialog;
@@ -99,7 +102,7 @@ namespace Imendio.Blam {
 
         private ChannelCollection mCollection;
 
-        private uint              mAutoRefreshId;
+        Timer                     AutoRefreshTimer;
 
         public static TargetEntry[] DragEntries = new TargetEntry[] {
             new TargetEntry("STRING", 0, (uint)TargetType.String),
@@ -142,6 +145,8 @@ namespace Imendio.Blam {
             SetupDBus();
 
             Gtk.Application.Init ();
+			Context = new GLib.GLibSynchronizationContext();
+			SynchronizationContext.SetSynchronizationContext(Context);
 
             Proxy.InitProxy ();
 
@@ -151,12 +156,9 @@ namespace Imendio.Blam {
                 Directory.CreateDirectory(Defines.APP_HOMEDIR);
             }
 
+            ItemStore.Load();
             mCollection = ChannelCollection.LoadFromFile (Defines.APP_HOMEDIR + "/collection.xml");
 
-            mCollection.ChannelUpdated         += ChannelUpdatedCb;
-            mCollection.ChannelAdded           += ChannelAddedCb;
-			mCollection.ChannelGroupAdded      += ChannelGroupAddedCb;
-            mCollection.ChannelRemoved         += ChannelRemovedCb;
             mCollection.ChannelRefreshStarted  += ChannelRefreshStartedCb;
             mCollection.ChannelRefreshFinished += ChannelRefreshFinishedCb;
 
@@ -169,7 +171,7 @@ namespace Imendio.Blam {
                 ShowNextUpdateTime();
             } else {
                 if(Conf.Get(Preference.REFRESH_AT_START, false) == true){
-                    mCollection.RefreshAllAsync();
+                    mCollection.RefreshAll();
                 }
             }
 
@@ -182,7 +184,7 @@ namespace Imendio.Blam {
                                                         "mainWindow", null);
             gladeXML.Autoconnect(this);
 
-            channelList = new ChannelList(mCollection.Channels, mCollection.Groups);
+            channelList = new ChannelList(mCollection);
             ((Container)channelListSw).Child = channelList;
 
             channelList.ChannelSelectedEvent   += ChannelSelected;
@@ -191,7 +193,10 @@ namespace Imendio.Blam {
             channelList.RemoveChannelEvent     += RemoveChannelActivated;
             channelList.RefreshChannelEvent    += RefreshChannelActivated;
 
-            itemView = new ItemView ();
+			itemList = new ItemList(itemView, channelList);
+			((Container)itemListSw).Child = itemList;
+
+            itemView = new ItemView(itemList);
 
             Frame f = new Frame ();
             f.Shadow = ShadowType.In;
@@ -199,14 +204,11 @@ namespace Imendio.Blam {
             itemPaned.Add2 (f);
             f.Show ();
             itemView.OnUrl += OnUrl;
-            //itemView.Widget.DomMouseClick += new Gecko.DomMouseClickHandler(OnButtonPressEvent);
 
-            itemList = new ItemList(itemView);
-            ((Container)itemListSw).Child = itemList;
+			// a bit silly to do it every time, but works
+			itemList.PropertyChanged += (sender, e) => printMenuItem.Sensitive = true;
 
-            itemList.ItemSelected += ItemSelected;
-
-            trayIcon = new TrayIcon (Catalog.GetString ("Blam News Reader"));
+            trayIcon = new TrayIcon (Catalog.GetString ("Blam News Reader"), mCollection);
             trayIcon.ButtonPressEvent += TrayIconButtonPressCb;
             trayIcon.RefreshAllEvent += RefreshAllActivated;
             trayIcon.PreferencesEvent += PreferencesActivated;
@@ -241,7 +243,7 @@ namespace Imendio.Blam {
             addGroupDialog = new AddGroupDialog (this);
             preferencesDialog = new PreferencesDialog (this.Window);
             opmlDialog = new OpmlDialog (this.Window);
-            opmlDialog.ChannelAdded += mCollection.Add;
+            opmlDialog.ChannelAdded += channel => mCollection.Add(channel, true);
             opmlDialog.ImportFinished += OpmlImportFinished;
 
         }
@@ -331,8 +333,6 @@ namespace Imendio.Blam {
                 return;
             }
 
-            itemList.CurrentChannel = channel;
-
             blogNameLabel.Markup = "<b>" + channel.Name + "</b>";
             mainWindow.Title = "Blam - " + channel.Name;
 
@@ -348,12 +348,6 @@ namespace Imendio.Blam {
             editEntryKeywordsMenuItem.Sensitive = sensitive;
         }
 
-        private void ItemSelected(Imendio.Blam.Item item)
-        {
-            itemView.CurrentItem = item;
-            printMenuItem.Sensitive = true;
-        }
-
         public void MarkEntryAsUnreadActivated (object obj, EventArgs args)
         {
             Item item = itemList.GetSelected ();
@@ -362,7 +356,8 @@ namespace Imendio.Blam {
             }
 
             // Toggle unread status
-            item.SetUnread (!item.Unread);
+            item.Unread = !item.Unread;
+            itemList.EmitSelectedRowChanged();
         }
 
         public void EditEntryKeywordsActivated (object obj, EventArgs args)
@@ -573,18 +568,17 @@ namespace Imendio.Blam {
             using (var diag = new AddChannelDialog(TheApp.Window)) {
                 var response = diag.Run();
 
+                diag.Hide();
                 switch (response) {
                 case (int)ResponseType.Ok:
                     var chan = new Channel();
                     chan.Url = diag.Url;
                     chan.http_username = diag.Username;
                     chan.http_password = diag.Password;
-                    TheApp.CCollection.Add(chan);
+                    TheApp.CCollection.Add(chan, true);
                     break;
                 }
-
-                diag.Hide();
-            }
+			}
         }
 
         public void AddGroupActivated (object obj, EventArgs args)
@@ -649,7 +643,7 @@ namespace Imendio.Blam {
         public void RefreshChannelActivated(IChannel channel)
         {
             if (channel != null) {
-                channel.RefreshAsync(null);
+                channel.RefreshAsync();
             }
         }
 
@@ -665,7 +659,7 @@ namespace Imendio.Blam {
             /* First move the refresh back */
             StartStopAutoRefresh();
             /* And pretend a timeout occurred */
-            TimeoutRefreshAll();
+            DoRefreshAll();
         }
 
         private void DragDataReceivedCb(object o, DragDataReceivedArgs args)
@@ -696,28 +690,6 @@ namespace Imendio.Blam {
             }
         }
 
-        private void ChannelAddedCb (IChannel channel)
-        {
-            channelList.Add (channel);
-        }
-
-		private void ChannelGroupAddedCb(IChannel group, IChannel chan)
-		{
-			channelList.AddToGroup(group, chan);
-		}
-
-        private void ChannelUpdatedCb (IChannel channel)
-        {
-            channelList.Updated (channel);
-            UpdateTotalNumberOfUnread ();
-        }
-
-        private void ChannelRemovedCb (IChannel channel)
-        {
-            channelList.Remove (channel);
-            UpdateTotalNumberOfUnread ();
-        }
-
         private void ChannelRefreshStartedCb (IChannel channel)
         {
             uint contextId = statusbar.GetContextId("update-status");
@@ -733,7 +705,6 @@ namespace Imendio.Blam {
             uint contextId = statusbar.GetContextId("update-status");
 
             statusbar.Pop (contextId);
-            channelList.Updated (channel);
             UpdateTotalNumberOfUnread ();
 
             if (channelList.GetSelected () == channel) {
@@ -756,33 +727,25 @@ namespace Imendio.Blam {
             statusbar.Push(contextId, StatusString);
         }
 
-        private bool TimeoutRefreshAll ()
+		void DoRefreshAll(object obj = null)
         {
             ShowNextUpdateTime();
-            mCollection.RefreshAllAsync();
-
-            /* Continue until source is removed */
-            return true;
+            mCollection.RefreshAll();
         }
 
-        private void StartStopAutoRefresh ()
-        {
-            bool doAutoRefresh;
+		private void StartStopAutoRefresh ()
+		{
+			bool doAutoRefresh = Conf.Get (Preference.AUTO_REFRESH, false);
+			if (!doAutoRefresh)
+				return;
 
-            if (mAutoRefreshId != 0) {
-                GLib.Source.Remove (mAutoRefreshId);
-                mAutoRefreshId = 0;
-            }
+			var rate = TimeSpan.FromMinutes(Conf.Get(Preference.AUTO_REFRESH_RATE, 15));
+			if (AutoRefreshTimer != null) {
+				AutoRefreshTimer.Change(rate, rate);
+			}
 
-            doAutoRefresh = Conf.Get (Preference.AUTO_REFRESH, false);
-            if (!doAutoRefresh) {
-                return;
-            }
-
-            // Check if any feeds require refresh every ten seconds
-            mAutoRefreshId = GLib.Timeout.Add ((uint) Conf.Get(Preference.AUTO_REFRESH_RATE, 15) * 60 * 1000,
-                                               new GLib.TimeoutHandler (TimeoutRefreshAll));
-        }
+			AutoRefreshTimer = new Timer(DoRefreshAll, null, rate, rate);
+		}
 
         private void ConfNotifyHandler (object sender, NotifyEventArgs args)
         {
@@ -858,7 +821,7 @@ namespace Imendio.Blam {
             Conf.Sync ();
         }
 
-        private void UpdateTotalNumberOfUnread ()
+        public void UpdateTotalNumberOfUnread()
         {
             int nrOfUnread, nrOfNew;
 
@@ -869,14 +832,6 @@ namespace Imendio.Blam {
                                                  channelsLabelText +
                                                  "</b>",
                                                  nrOfUnread);
-
-            /* Total number of unread items */
-            string str = string.Format (Catalog.GetPluralString ("{0} unread item", "{0} unread items", nrOfUnread),
-                                             nrOfUnread);
-            str += " ";
-            /* Number of new (not-skipped-over) entries. Gets appended to previous string */
-            str += string.Format(Catalog.GetPluralString("({0} new)", "({0} new)", nrOfNew), nrOfNew);
-            trayIcon.Tooltip = str;
         }
 
 	public static void SetProcessName(string name)
